package GCDisplay;

###################################################
#
#  Copyright 2005 Tian
#
#  This file is part of GCfilms.
#
#  GCfilms is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  GCfilms is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with GCfilms; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###################################################

use strict;

{
    package GCFilterAll;

    sub new
    {
        my $proto = shift;
        my $class = ref($proto) || $proto;
        my $self  = {};
        bless ($self, $class);
        return $self;
    }

    sub test
    {
        return 1;
    }
}

{
    package GCFilterNotSeen;

    sub new
    {
        my $proto = shift;
        my $class = ref($proto) || $proto;
        my $self  = {};
        bless ($self, $class);
        return $self;
    }

    sub test
    {
        my ($self, $info) = @_;

        return (! $info->{seen});
    }
}

{
    package GCFilterType;

    sub new
    {
        my ($proto, $type) = @_;
        my $class = ref($proto) || $proto;
        my $self  = {};

        $self->{type} = $type;

        bless ($self, $class);
        return $self;
    }

    sub test
    {
        my ($self, $info) = @_;

        my $type = $self->{type};

        return (($info->{type} =~ /^$type,/) || ($info->{type} =~ /,$type,/));
    }
}

{
    package GCFilterRating;

    sub new
    {
        my ($proto, $rating) = @_;
        my $class = ref($proto) || $proto;
        my $self  = {};

        $self->{rating} = $rating;

        bless ($self, $class);
        return $self;
    }

    sub test
    {
        my ($self, $info) = @_;

        return ($info->{rating} >= $self->{rating});
    }
}

{
    package GCFilterSearch;

    sub new
    {
        my ($proto, $info) = @_;
        my $class = ref($proto) || $proto;
        my $self  = {};

        $self->{info} = $info;

        bless ($self, $class);
        return $self;
    }

    sub test
    {
        my ($self, $info) = @_;

        return 0 if ($self->{info}->{seen} != -1) && ($info->{seen} != $self->{info}->{seen});
        return 0 if $self->{info}->{title} && ($info->{title} !~ m/$self->{info}->{title}/i);
        return 0 if $self->{info}->{director} && ($info->{director} !~ m/$self->{info}->{director}/i);
        return 0 if $self->{info}->{type} && ($info->{type} !~ m/$self->{info}->{type}/i);

        return 0 if $self->{info}->{borrower} && ($info->{borrower} ne $self->{info}->{borrower});

        return 0 if $self->{info}->{rating} && ($info->{rating} < $self->{info}->{rating});

        if ($self->{info}->{actors})
        {
            my @actors = split m/,/, $self->{info}->{actors};
            foreach (@actors)
            {
                s/^\s*//;
                s/\s*$//;
                return 0 if $info->{actors} !~ /$_/i;
            }
        }
        
        return 1;
    }
}

{
    package GCFilterBorrowing;

    sub new
    {
        my ($proto, $borrowed) = @_;
        my $class = ref($proto) || $proto;
        my $self  = {};
        
        $self->{borrowed} = $borrowed;
        
        bless ($self, $class);
        return $self;
    }

    sub test
    {
        my ($self, $info) = @_;

        return (($info->{borrower} eq 'none') || !$info->{borrower}) 
            if ! $self->{borrowed};
        return (($info->{borrower} ne 'none') && $info->{borrower}) 
    }
}


use Gtk2;

{
    package GCRatingDialog;
    use base "Gtk2::Dialog";

    sub getUserValue
    {
        my $self = shift;
        my $value = -1;
        my $code = $self->run;
        $value = $self->{rating}->get_value if ($code eq 'ok');
        $self->hide;
        return $value;
    }

    sub new
    {
        my ($proto, $parent) = @_;
        my $class = ref($proto) || $proto;
        my $self  = $class->SUPER::new($parent->{lang}->{RatingTitle},
                              $parent,
                              [qw/modal destroy-with-parent/],
                              'gtk-ok' => 'ok',
                              'gtk-cancel' => 'cancel'
                              );

        my $label = Gtk2::Label->new($parent->{lang}->{RatingLabel});
        $label->set_line_wrap(1);
        $label->set_padding(5,0);
        my  $adj = Gtk2::Adjustment->new(5, 0, 10, 1, 1, 0) ;
        $self->{rating} = Gtk2::SpinButton->new($adj, 0, 0);

        my $hboxRating = new Gtk2::HBox(1,10);

        $self->vbox->set_homogeneous(0);
        $self->vbox->set_spacing(20);
        $self->vbox->pack_start($label, 0, 0, 5);
        $hboxRating->pack_start($self->{rating}, 0, 0, 5);
        $self->vbox->pack_start($hboxRating, 0, 0, 5);
        $self->vbox->show_all;

        bless ($self, $class);
        return $self;
    }
}

{
    package GCSearchDialog;
    use base "Gtk2::Dialog";
    
    sub show
    {
        my $self = shift;
        
        $self->setTypes($self->{parent}->{movies}->getTypes);
        my @borrowers = split m/\|/, $self->{parent}->{options}->borrowers;
        $self->setBorrowers(\@borrowers);
        
        $self->SUPER::show();
        $self->show_all;
        $self->{search} = undef;
        my $ended = 0;
        while (!$ended)
        {
            my $response = $self->run;
            if ($response eq 'ok')
            {
                my %info;
            
                $info{title} = $self->{title}->get_text;
                $info{rating} = $self->{rating}->get_value;
                $info{director} = $self->{director}->get_text;
                $info{actors} = $self->{actors}->get_text;

                my @children = $self->{type}->get_children;
                $info{type} = $children[0]->get_text;
            
                my $seen = -1;
                foreach (@{$self->{seen}})
                {
                    last if $_->get_active;
                    $seen++;
                }
                $info{seen} = $seen;
            
                $info{borrower} = $self->{borrowerOption}->child->get_label;
                $info{borrower} =  'none' if $self->{borrowerOption}->child->get_label eq $self->{parent}->{lang}->{PanelNobody};

                $self->{search} = \%info;
            }
            $ended = 1 if ($response eq 'ok') || ($response eq 'cancel') || ($response eq 'delete-event');
            $self->clear if ($response eq 'reject');
        }
        $self->hide;
    }
    
    sub clear
    {
        my $self = shift;
        
        $self->{title}->set_text('');
        $self->{rating}->set_value(0);
        $self->{director}->set_text('');
        $self->{actors}->set_text('');
        $self->{seen}[0]->set_active(1);
        
        $self->{borrowerOption}->remove_menu;
        $self->{borrower}->set_active(0) if $self->{borrower};
        $self->{borrowerOption}->set_menu($self->{borrower}) if $self->{borrower};
        
        my @children = $self->{type}->get_children;
        $children[0]->set_text('');
    }
    
    sub search
    {
        my $self = shift;
        
        return $self->{search};
    }

    sub setTypes
    {
        my ($self, $values) = @_;

        my $current = ($self->{type}->get_children)[0]->get_text;
        $self->{type}->set_popdown_strings(@$values);
        ($self->{type}->get_children)[0]->set_text($current);
    }
    
    sub setBorrowers
    {
        my ($self, $values) = @_;
        
        #my $current = $self->borrower if $self->{borrower};
        
        my @borrowers = ('', $self->{parent}->{lang}->{PanelNobody}, @{$values}, $self->{parent}->{lang}->{PanelUnknown});

        $self->{borrowerOption}->remove_menu;
        $self->{borrower}->destroy if $self->{borrower};
        $self->{borrower} = new Gtk2::Menu();

        foreach (@borrowers)
        {
            my $item = Gtk2::MenuItem->new_with_label($_);
		    $self->{borrower}->append($item);
        }
        $self->{borrowers} = \@borrowers;
		$self->{borrowerOption}->set_menu($self->{borrower});
        
        #$self->borrower($current) if $self->{borrower};
        
        $self->{borrowerOption}->show_all;
    }

    
    sub new
    {
        my ($proto, $parent) = @_;
        my $class = ref($proto) || $proto;
        my $self  = $class->SUPER::new($parent->{lang}->{SearchTitle},
                              $parent,
                              [qw/modal destroy-with-parent/],
                              'gtk-clear' => 'reject',
                              'gtk-ok' => 'ok',
                              'gtk-cancel' => 'cancel'
                              );

        $self->{parent} = $parent;
                              
        my $table = new Gtk2::Table(9,2,0);
        $table->set_row_spacings(10);
        $table->set_col_spacings(10);
        $table->set_border_width(20);
                              
        my $titleLabel = new Gtk2::Label($parent->{lang}->{SearchMovieTitle});
        $titleLabel->set_alignment(0,0.5);
        $self->{title} = new Gtk2::Entry;
        $table->attach($titleLabel, 0, 1, 0, 1, 'fill', 'fill', 0, 0);
        $table->attach($self->{title}, 1, 2, 0, 1, 'fill', 'expand', 0, 0);

        my $directorLabel = new Gtk2::Label($parent->{lang}->{SearchDirector});
        $directorLabel->set_alignment(0,0.5);
        $self->{director} = new Gtk2::Entry;
        $table->attach($directorLabel, 0, 1, 1, 2, 'fill', 'fill', 0, 0);
        $table->attach($self->{director}, 1, 2, 1, 2, 'fill', 'expand', 0, 0);

        my $typeLabel = new Gtk2::Label($parent->{lang}->{SearchType});
        $typeLabel->set_alignment(0,0.5);
        $self->{type} = Gtk2::Combo->new;
        ($self->{type}->get_children)[0]->set_editable(0);
        $table->attach($typeLabel, 0, 1, 2, 3, 'fill', 'fill', 0, 0);
        $table->attach($self->{type}, 1, 2, 2, 3, 'fill', 'expand', 0, 0);
        
        my $actorsLabel = new Gtk2::Label($parent->{lang}->{SearchActors});
        $actorsLabel->set_alignment(0,0.5);
        $self->{actors} = new Gtk2::Entry;
        $table->attach($actorsLabel, 0, 1, 3, 4, 'fill', 'fill', 0, 0);
        $table->attach($self->{actors}, 1, 2, 3, 4, 'fill', 'expand', 0, 0);

        $table->attach(Gtk2::HSeparator->new, 0, 2, 4, 5, 'fill', 'fill', 0, 0);
        
        my $hboxSeen = new Gtk2::HBox(0,0);
        my $labelSeen = new Gtk2::Label($parent->{lang}->{SearchSeen});
		$self->{seen}[0] = new  Gtk2::RadioButton(undef,$parent->{lang}->{SearchSeenUndef});
		my $seenGroup = $self->{seen}[0]->get_group;
        $self->{seen}[1] = new  Gtk2::RadioButton($seenGroup,$parent->{lang}->{SearchSeenNo});
        $self->{seen}[2] = new  Gtk2::RadioButton($seenGroup,$parent->{lang}->{SearchSeenYes});
 		$hboxSeen->pack_start($self->{seen}[0],0,0,10);
		$hboxSeen->pack_start($self->{seen}[1],0,0,10);
		$hboxSeen->pack_start($self->{seen}[2],0,0,10);
        $table->attach($labelSeen, 0, 1, 5, 6, 'fill', 'fill', 0, 0);
        $table->attach($hboxSeen, 1, 2, 5, 6, 'fill', 'fill', 0, 0);
        
        my $ratingLabel = new Gtk2::Label($parent->{lang}->{SearchRating});
        $ratingLabel->set_alignment(0,0.5);
        my  $adj = Gtk2::Adjustment->new(0, 0, 10, 1, 1, 0) ;
        $self->{rating} = Gtk2::SpinButton->new($adj, 0, 0);
        $table->attach($ratingLabel, 0, 1, 6, 7, 'fill', 'fill', 0, 0);
        $table->attach($self->{rating}, 1, 2, 6, 7, 'fill', 'expand', 0, 0);

        $table->attach(Gtk2::HSeparator->new, 0, 2, 7, 8, 'fill', 'fill', 0, 0);
        
        my $borrowerLabel = new Gtk2::Label($parent->{lang}->{PanelBorrower});
        $self->{borrowerOption} = new Gtk2::OptionMenu();
        $table->attach($borrowerLabel, 0, 1, 8, 9, 'fill', 'fill', 0, 0);
        $table->attach($self->{borrowerOption}, 1, 2, 8, 9, 'fill', 'expand', 0, 0);
        
        $self->vbox->pack_start($table, 1, 1, 5);
        $self->vbox->show_all;

        bless ($self, $class);
        return $self;
    }
}


1;