package GCMainWindow;

###################################################
#
#  Copyright 2005 Tian
#
#  This file is part of GCfilms.
#
#  GCfilms is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  GCfilms is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with GCfilms; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###################################################

    
    
use strict;
use Gtk2;

my $VERSION = '4.8';

{
    package GCMovieWindow;
    use base "Gtk2::Dialog";
    use GCGraphicComponents;

    sub new
    {
        my ($proto, $parent) = @_;
        my $class = ref($proto) || $proto;
        my $self  = $class->SUPER::new('',
                              undef,
                              [qw/no-separator destroy-with-parent/]);
        bless ($self, $class);
        
        $self->set_modal(0);
        $self->set_icon_from_file($ENV{GCF_SHARE_DIR}.'/icons/gcfilms_logo32.png');
        $self->set_title('GCfilms - '.$parent->{movies}->currentTitle);

        $self->{panel} = new GCMoviePanel($parent, $parent->{options}, 1);
        
        my $scrollPanelMovie = new Gtk2::ScrolledWindow;
        $scrollPanelMovie->set_policy ('automatic', 'automatic');
        $scrollPanelMovie->set_shadow_type('etched-in');
        $scrollPanelMovie->add_with_viewport($self->{panel});

        $self->vbox->add($scrollPanelMovie);

        $self->vbox->show_all;
        $self->{panel}->setShowOption($parent->{displayOptionsDialog}->{show});

        $self->set_default_size(600,500);
        
        return $self;
    }
}

{
    package GCFrame;
    use base "Gtk2::Window";

    use GCMoviesLists;
    use GCPlugins;
    use GCGraphicComponents;
    use GCUtils;
    use GCOptions;
    use GCBorrowings;
    use GCDialogs;
    use GCExport;
    use GCImport;
    use GCSplash;
    use GCLang;
    use GCDisplay;
    use GCData;

    use LWP;
    use URI::Escape;
    use File::Basename;
    use IO::Handle;
    use Storable qw(store_fd fd_retrieve);

    sub beforeDestroy
    {
		my $self = shift;
		$self->leave;
		return 1;
    }

    sub leave
    {
        my $self = shift;

		if ($self->{options}->autosave)
		{
		    $self->saveList;
		}
	
		my ($width, $height) = $self->get_size;
	
		$self->{options}->width($width);
		$self->{options}->height($height);
		$self->{options}->split($self->{pane}->get_position) if ($self->{pane});
	
		$self->{options}->save;

        $self->{movies}->clean;

        if ($^O !~ /win32/i)
        {
            close $self->{searchJob}->{reader};
            close $self->{searchJob}->{writer};
            kill 9, $self->{searchJob}->{pid};
            wait;
        }
        
        $self->destroy;
    }

    sub deleteCurrentMovie
    {
        my $self = shift;
		my $response = 'yes';
	
		if ($self->{options}->confirm)
		{
		    my  $dialog = Gtk2::MessageDialog->new($self,
							   [qw/modal destroy-with-parent/],
							   'question',
							   'yes-no',
							   $self->{lang}->{RemoveConfirm});
		    $response = $dialog->run;
		    $dialog->destroy;
		}
	
		if ($response eq 'yes')
		{
		    $self->{movies}->removeCurrentMovie;
		}
    }

    sub newMovie
    {
        my ($self, $self2) = @_;
        $self = $self2 if $self2;

		my $info = {};
		$info->{title} = $self->{lang}->{NewMovie};
		$info->{seen} = 0;
		$self->addMovie($info, 1);
    }

    sub loadUrl
    {
        my ($self, $url) = @_;
        
        my $baseUrl;
        my $plugin;
        foreach (values %GCPlugins::pluginsMap)
        {
            ($baseUrl = $_->getMovieUrl) =~ s/http:\/\/(w{3})?//;
            $plugin = $_;
            last if $url =~ m/$baseUrl/;
        }
        return unless $url =~ m/$baseUrl/;
        $self->setWaitCursor($self->{lang}->{StatusGetInfo});
        $self->newMovie;
        my $info = $plugin->loadUrl($url);
        $self->restoreCursor;
        return if ! $info->{title};        
        $self->addMovie($info, 0);
    }
    
    sub searchMovie
    {
        my ($self, $title, $forceask, $currentPlugin) = @_;

        my $plugin;
        
        if (($self->{options}->plugin eq 'multi') && !$forceask)
        {
            my $pluginName = $self->{multiSiteDialog}->getPlugin($currentPlugin);
            $plugin = $GCPlugins::pluginsMap{$pluginName};
            if (!$plugin)
            {
              my $dialog = Gtk2::MessageDialog->new($self,
							   [qw/modal destroy-with-parent/],
							   'error',
							   'ok',
							   $self->{lang}->{MultiSiteEmptyError});
		      my $response = $dialog->run;
		      $dialog->destroy;
		      return;
            
            }
        }
        elsif ($self->{options}->plugin eq 'multiask')
        {
            $self->{pluginsAskDialog}->setPluginsList($self->{options}->multisite);
            $self->{pluginsAskDialog}->query($title);
            $self->{pluginsAskDialog}->show;
            $plugin = $self->{pluginsAskDialog}->plugin;
            return if !$plugin;
            $plugin->setProxy($self->{options}->proxy);
            $title = $self->{pluginsAskDialog}->query;
        }
        elsif ($forceask || ! $self->{plugin})
        {
            $self->{pluginsDialog}->query($title);
            $self->{pluginsDialog}->show;
            $plugin = $self->{pluginsDialog}->plugin;
            return if !$plugin;
            $plugin->setProxy($self->{options}->proxy);
            $title = $self->{pluginsDialog}->query;
        }
        else
        {
            $plugin = $self->{plugin};
        }

        $plugin->{title} = $title;
        $plugin->{type} = 'load';
        
        if (! $self->{options}->searchStop)
        {
            $self->setWaitCursor($self->{lang}->{StatusSearch}.' ('.$plugin->getName.')');
            $plugin->load;
            $self->restoreCursor;
        }
        else
        {
            store_fd $plugin, $self->{searchJob}->{command};
            my $getInfo = 0;
            my $dialog = Gtk2::MessageDialog->new($self,
                [qw/modal destroy-with-parent/],
                'info',
                'cancel',
                $self->{lang}->{StatusSearch} . "\n" . $plugin->getName);
            my $watch = Glib::IO->add_watch(fileno($self->{searchJob}->{data}),
                                'in',
                                sub {
                                    return if !$dialog;
                                    $dialog->response('cancel');
                                    $getInfo = 1;
                                    return 0;
                                });
            $dialog->run if !$getInfo;
            Glib::Source->remove($watch);
            $dialog->destroy;
            my $command = $self->{searchJob}->{command};
            if ($getInfo)
            {
                $plugin = fd_retrieve($self->{searchJob}->{data});
                print $command "OK\n";
            }
            else
            {
                print $command "STOP\n";
                return;
            }
        }
            
		my $movieNumber = $plugin->getMoviesNumber();
	
		if ($movieNumber == 0)
		{
		    my $force = 0;
		    my $idx = 0;
		    if (!$forceask && ($self->{options}->plugin eq 'multi')
		     && (($currentPlugin + 1) < $self->{multiSiteDialog}->getPluginsNumber))
		    {
		      $idx = $currentPlugin + 1;
		    }
		    else
		    {
		      my  $dialog = Gtk2::MessageDialog->new($self,
							   [qw/modal destroy-with-parent/],
							   'error',
							   'yes-no',
							   $self->{lang}->{NoMovieFound});
		      my $response = $dialog->run;
		      $dialog->destroy;
		      return if $response ne 'yes';
		      $force = 1;
		    }
		    
            $self->searchMovie($title,$force, $idx);
		}
		elsif ($movieNumber == 1)
		{
		    $self->getMovieInfo($plugin, 0);
		}
		else
		{
		    my $withNext = 0;
            $withNext = 1 if ($self->{options}->plugin eq 'multi')
		        && (($currentPlugin + 1) < $self->{multiSiteDialog}->getPluginsNumber);
		    
		    my @movies = $plugin->getMovies();
            $self->{resultsDialog}->setWithNext($withNext);
            $self->{resultsDialog}->setSearchSite($plugin->getName);
		    $self->{resultsDialog}->setList(@movies);
		    my $next = $self->{resultsDialog}->show;
		    if ($self->{resultsDialog}->{validated})
		    {
				my $idx = $self->{resultsDialog}->{movieIndex};
				$self->getMovieInfo($plugin, $idx);
		    }
		    elsif ($next)
		    {
		      $self->searchMovie($title,0, $currentPlugin + 1);
		    }
		}
    }

    sub getMovieInfo
    {
        my ($self, $plugin, $idx) = @_;
        
        $plugin->{wantedIdx} = $idx;
        $plugin->{type} = 'info';
        my $info;
        
        if (! $self->{options}->searchStop)
        {
            $self->setWaitCursor($self->{lang}->{StatusGetInfo});
            $info = $plugin->getMovieInfo;
            $self->restoreCursor;
        }
        else
        {
            store_fd $plugin, $self->{searchJob}->{command};
            my $getInfo = 0;
            my $dialogGet = Gtk2::MessageDialog->new($self,
                [qw/modal destroy-with-parent/],
                'info',
                'cancel',
                $self->{lang}->{StatusGetInfo});
            my $watch = Glib::IO->add_watch(fileno($self->{searchJob}->{data}),
                                'in',
                                sub {
                                    return if !$dialogGet;
                                    $dialogGet->response('cancel');
                                    $getInfo = 1;
                                    return 0;
                                });
            $dialogGet->run if !$getInfo;
            Glib::Source->remove($watch);
            $dialogGet->destroy;
            my $command = $self->{searchJob}->{command};
            if ($getInfo)
            {
                $info = fd_retrieve($self->{searchJob}->{data});
                print $command "OK\n";
            }
            else
            {
                print $command "STOP\n";
                return;
            }
        }

        
        $info->{image} = '' if $info->{image} eq 'empty';        
        
        if ($self->{options}->askImport)
        {
            $self->{importFieldsDialog}->info($info);
            return if ! $self->{importFieldsDialog}->show;
            $info = $self->{importFieldsDialog}->info;
        }
        $self->addMovie($info, 0);
    }
    
    sub getUniqueImageFileName
    {
        my ($self, $suffix) = @_;
    
        if ( ! -e $self->{options}->images)
        {
            mkdir($self->{options}->images);
        }

        my $filePrefix = $self->{options}->images.$self->{imagePrefix};
        my @tmp_filenames = glob $filePrefix."*.*";
        my %modtimes;
        my @sorted = sort { ($modtimes{$a} ||= -M $a) <=>
                         ($modtimes{$b} ||= -M $b)
                       } @tmp_filenames;
        my $i = 0;
        my $mostRecent = $sorted[$i];
        while (! $mostRecent =~ /^[0-9]*\./ )
	    {
			$mostRecent = $sorted[$i];
			$i++;
	    }
        my $sysPrefix = $filePrefix;
        $sysPrefix =~ s/\\/\\\\/g if ($^O =~ /win32/i);
        $mostRecent =~ s/$sysPrefix([0-9]*)\..*/$1/;

        $mostRecent = 0 if !$mostRecent;

        #my ($name,$path,$suffix) = fileparse($info->{image}, "\.gif", "\.jpg", "\.png");

        my $picture = $filePrefix.$mostRecent.$suffix;

        while (-e $picture)
        {
            $mostRecent++;
            $picture = $filePrefix.$mostRecent.$suffix;
        }
        
        return $picture;
    }
    
    sub addMovie
    {
        my ($self, $info, $newMovie) = @_;

        $self->{movies}->updateCurrent if $newMovie;

        my $ignore = $self->{ignoreString};
        $self->{panel}->clear if $newMovie;
        
        $self->{panel}->title($info->{title}) if $info->{title} ne $ignore;
        $self->{panel}->date($info->{date}) if $info->{date} ne $ignore;
        $self->{panel}->time($info->{time}) if $info->{time} ne $ignore;
        $self->{panel}->director($info->{director}) if $info->{director} ne $ignore;
        $self->{panel}->nationality($info->{nat}) if $info->{nat} ne $ignore;
        $self->{panel}->orig($info->{orig}) if $info->{orig} ne $ignore;
        $self->{panel}->actors($info->{actors}) if $info->{actors} ne $ignore;
        $self->{panel}->synopsis($info->{synopsis}) if $info->{synopsis} ne $ignore;
        $self->{panel}->url($info->{url}) if $info->{url} ne $ignore;

        $self->{panel}->selectTitle if $newMovie;

        if ($info->{image} && ($info->{image} ne $ignore))
        {
            $self->setWaitCursor($self->{lang}->{StatusGetImage});
            my $browser = LWP::UserAgent->new;
            $browser->proxy(['http'], $self->{options}->proxy);
            $browser->agent($self->{agent});
  
            my ($name,$path,$suffix) = fileparse($info->{image}, "\.gif", "\.jpg", "\.png");
            my $picture = $self->getUniqueImageFileName($suffix);

            $browser->get($info->{image}, ':content_file' => $picture);

            $info->{image} = $picture;
            $self->{moviesList}->changeCurrent($info->{image})
                if ($info->{image} ne $self->{panel}->image);
            $self->{panel}->image($info->{image});
            $self->{movies}->markToBeAdded($picture);
            $self->restoreCursor;
        }

        if ($info->{type} ne $ignore)
        {
            $self->{panel}->setTypes($self->{movies}->getTypes);
            $self->{menubar}->setTypes($self->{movies}->getTypes);
            $self->{panel}->type($info->{type});
        }

        if ($newMovie)
        {
            $self->{movies}->addMovie($info);
        }
        else
        {
            $self->{movies}->updateCurrent;
            $self->{moviesList}->showCurrent;
        }
    }

    sub openMovie
    {
        my ($self, $filename) = @_;

        if ($self->{options}->autosave)
        {
            $self->saveList();
        }
        (my $history = $self->{options}->history) =~ s/(.*\|)?($filename\|)(.*\|)?/$2$1$3/;
        $history =~ s/(.*?)\|[^\|]*?$/$filename\|$1/ if $history !~ /^$filename/;
        $self->{options}->history($history);
        $self->{menubar}->{menuHistoryItem}->remove_submenu();
        $self->{menubar}->{menuHistory} = Gtk2::Menu->new();
        $self->{menubar}->addHistoryMenu();
        $self->{options}->file($filename);
        $self->{options}->save;
        $self->{movies}->load($self->{options}->file);
        $self->{movies}->display(0);
        $self->{movies}->select(0);
        $self->set_title('GCfilms - ['.basename($filename).']');			
    }

    sub openList
    {
        my $self = shift;
        my $fileDialog = new GCFileChooserDialog($self->{lang}->{OpenList}, $self, 'open');

        $fileDialog->set_filename($self->{options}->file);
        my $response = $fileDialog->run;
        if ($response eq 'ok')
        {
            my $fileName = $fileDialog->get_filename;
            $fileDialog->destroy;
            $self->openMovie($fileName);
        }
        else
        {
            $fileDialog->destroy;
        }
    }

    sub newList
    {
        my $self = shift;
        if ($self->{options}->autosave)
        {
            $self->saveList();
        }
        $self->{movies}->clearList;
        $self->{options}->file('');
        $self->set_title('GCfilms - []');
    }

    sub saveAs
    {
        my $self = shift;
        my $fileDialog = new GCFileChooserDialog($self->{lang}->{SaveList}, $self, 'save');

        $fileDialog->set_filename($self->{options}->file);
		my $response = $fileDialog->run;
		if ($response eq 'ok')
		{
				my $filename = $fileDialog->get_filename;
				my $file = $self->{options}->file;
				(my $history = $self->{options}->history) =~ s/(.*\|)?$file\|(.*\|)?/$filename\|$1$2/;
 				$self->{options}->history($history);
				$self->{menubar}->{menuHistoryItem}->remove_submenu();
				$self->{menubar}->{menuHistory} = Gtk2::Menu->new();
				$self->{menubar}->addHistoryMenu();
	      $self->{options}->file($filename);
		    $self->saveList();
		    $self->{options}->save;
	      $self->set_title('GCfilms - ['.basename($filename).']');
		}
        $fileDialog->destroy;
    } 

    sub saveList
    {
        my ($self, $self2) = @_;
        $self = $self2 if $self2;
				my $response = 'yes';

        if ($self->{options}->file)
        {
            $self->{movies}->save;
        }
        else
        {
	    my $dialog = Gtk2::MessageDialog->new($self,
                                                  [qw/modal destroy-with-parent/],
                                                  'info',
                                                  'yes-no',
                                                  $self->{lang}->{SaveCurrentList});
	    $response = $dialog->run();
	    $dialog->destroy ;
			$self->saveAs if ($response eq 'yes');
        }
    }

    sub removeSearch
    {
        my $self = shift;
        
        $self->setFilter('search', 'all');
    }
    
    sub search
    {
        my ($self, $self2, $value) = @_;
        $self = $self2 if ($self2 ne 'all') && ($self2 ne 'displayed');
        my $type = 'all';
        $type = $value if ($self != $self2);
        
        $self->{searchDialog}->show;
        
        my $info = $self->{searchDialog}->search;
        return if ! $info;
        
        $self->{menubar}->selectAll if $type eq 'all';
        $self->setFilter('search', $info);
    }
    
    sub showBorrowed
    {
        my $self = shift;
        $self->{movies}->updateCurrent;
        $self->{borrowedDialog}->setList($self->{movies}->{movieArray});
        $self->{borrowedDialog}->show;
    }
    
    sub export
    {
        my ($self, $exporter) = @_;
        $self->{exportDialog}->setModule($exporter);
        $self->{exportDialog}->show;
    }
    sub import
    {
        my ($self, $importer) = @_;
        $self->{importDialog}->setModule($importer);
        $self->{importDialog}->show;
    }

    sub importWithDetect
    {
        my ($self, $file) = @_;
  
        $file =~ s/^file:\/\/(.*)\W*$/$1/;
        $file =~ s/.$//ms;
        (my $suffix = $file) =~ s/.*\.(\w+)\W*$/$1/;
        
        foreach my $importer(@GCImport::importersArray)
        {
            my $current = $importer->getSuffix;
            if ($current eq $suffix)
            {
                $self->setWaitCursor($self->{lang}->{StatusGetInfo});
                my %options;
                $options{parent} = $self;
                $options{newList} = 0;
                $options{file} = $file;
                $options{lang} = $self->{lang};
                $importer->process(\%options);
                $self->restoreCursor;
                return;
            }
        }
    }

    sub optionsError
    {
        my ($self, $type) = shift;

        my $msg;
        if ($type eq 'open')
        {
            $msg = $self->{lang}->{OptionsOpenError};
        }
        elsif ($type eq 'create')
        {
            $msg = $self->{lang}->{OptionsCreateError};
        }
        else
        {
            $msg = $self->{lang}->{OptionsSaveError};
        }
            
        my $dialog = Gtk2::MessageDialog->new($self,
                                              [qw/modal destroy-with-parent/],
                                              'error',
                                              'ok',
                                              $msg.$self->{options}->{file});
        $dialog->run();
        $dialog->destroy ;

        $self->destroy;
    }

    sub checkImagesDirectory
    {
        my ($self, $withDialog) = @_;
        my $error = 0;
        if ( ! -e $self->{options}->images)
        {
            $! = '';
            $error = 1 if (!mkdir($self->{options}->images));
            rmdir $self->{options}->images if ! $!;
        }
        else
        {
            $error = 1 if !( -d _ && -r _ && -w _ && -x _ )
        }
        return $error if !$withDialog;
        if ($error)
        {
            my $fileDialog = new GCFileChooserDialog($self->{lang}->{OptionsImagesTitle}, $self, 'select-folder');
            my  $errorDialog = Gtk2::MessageDialog->new($self,
						   [qw/modal destroy-with-parent/],
						   'error',
						   'ok',
						   $self->{lang}->{ImageError});
            $fileDialog->set_filename($self->{options}->images);
            my $response;
            do
            {
                $errorDialog->run();
                $errorDialog->hide();
                $response = '';
                $response = $fileDialog->run while ($response ne 'ok');
                $self->{options}->images($fileDialog->get_filename);
            } while ($self->checkImagesDirectory(0));
            $errorDialog->destroy;
            $fileDialog->destroy;
        }
        return $error;
    }

    sub setWaitCursor
    {
        print "Wait\n";
		my ($self, $message) = @_;
        $self->setStatus($message);
		$self->window->set_cursor(Gtk2::Gdk::Cursor->new('watch'));
		Gtk2->main_iteration while (Gtk2->events_pending);
    }
    sub restoreCursor
    {
        print "Restore\n";
		my $self = shift;
        $self->restoreStatus;
		$self->window->set_cursor(Gtk2::Gdk::Cursor->new('left_ptr'));
		Gtk2->main_iteration while (Gtk2->events_pending);
    }

    sub setFilter
    {
        my ($self, $filter, $parameter) = @_;
        $self->{movies}->updateCurrent;
        $self->{movies}->setFilter($filter, $parameter);
    }

    sub onSortClick
    {
        my ($widget, $self) = @_;
        $self->{moviesList}->getSortButton()->signal_connect('clicked', \&onSortClick, $self)
            if ($self->{moviesList}->setSortOrder($self->{movies}->changeOrder));
        ;
    }

    sub selectMultiPluginsList
    {
        my $self = shift;
        
        $self->{multiSiteDialog}->show;
    }

    sub options
    {
        my ($self, $self2) = @_;
        $self = $self2 if $self2;

        my $transform = $self->{options}->transform;
        my $articles = $self->{options}->articles;
        my $formats = $self->{options}->formats;
        my $urlPicture = $self->{options}->urlPicture;

        $self->{optionsDialog}->show;
        #$self->{multiSiteDialog}->show;

        $self->checkTransform
            if ($self->{options}->articles ne $articles) || ($self->{options}->transform != $transform);
            
        $self->checkFormats
            if ($self->{options}->formats ne $formats);

        $self->checkUrlPicture
            if ($self->{options}->urlPicture ne $urlPicture);
       
        $self->checkDisplayed;
        $self->checkPlugin;
        $self->checkView;
    }
    
    sub displayOptions
    {
        my ($self, $self2) = @_;
        $self = $self2 if $self2;

        my $displayed = $self->{options}->displayed;
        $self->{displayOptionsDialog}->show;
        $self->checkPanel
            if ($self->{options}->displayed ne $displayed);
    }

    sub borrowers
    {
        my ($self, $self2) = @_;
        $self = $self2 if $self2;

        $self->{borrowersDialog}->show;
        
        $self->checkBorrowers;
    }
    
    sub rename
    {
        my ($self, $self2) = @_;
        $self = $self2 if $self2;

        $self->{renameDialog}->show;
        
        my ($old, $new) = $self->{renameDialog}->names;
        return if ! $old;
        $self->{movies}->rename($old,$new);
    }

    sub about
    {
        my $self = shift;
        
        my $about = new GCAboutDialog($self, $VERSION);
        $about->show;
    }

    sub help
    {
        my ($self, $self2) = @_;
        $self = $self2 if $self2;

        $self->openUrl('http://home.gna.org/gcfilms/doc');
    }

    sub openUrl
    {
        my ($self, $url) = @_;

        my $command = $self->{options}->browser.' '.$url;
        if ($^O =~ /win32/i)
        {
            system "start $command";
        }
        else
        {
            system "$command &";
        }
    }

    sub setStatus
    {
		my ($self, $status) = @_;
		$self->{status}->push(1, $status) if ($self->{status});
    }
    
    sub restoreStatus
    {
        my $self = shift;
        
        $self->{status}->pop(1);
    }

    sub display
    {
        my ($self, $idx) = @_;
        
        $self->{movies}->display($idx);
    }
    
    sub displayInWindow
     {
        my ($self) = @_;

        my $window = new GCMovieWindow($self);
        
        $window->show;

        my @borrowers = split m/\|/, $self->{options}->borrowers;
        $window->{panel}->setBorrowers(\@borrowers);

        $self->{movies}->displayInPanel($window->{panel});
    }
   

    sub setMoviesList
    {
        my ($self, $view, $columns) = @_;
          
        $self->{pane}->remove($self->{moviesList}) if $self->{pane} && $self->{moviesList};
        $self->{moviesList}->destroy if $self->{moviesList};
        
        if ($view == 0)
        {
            $self->{moviesList} = new GCTextList($self, $self->{lang}->{Movies});
        }
        else
        {
            $self->{moviesList} = new GCImageList($self, $self->{lang}->{Movies}, $columns);
        }
        $self->{moviesList}->setSortOrder($self->{movies}->getSortOrder) if $self->{movies};
        $self->{pane}->pack1($self->{moviesList},1,0) if $self->{pane};
        $self->{moviesList}->show_all;
        $self->{moviesList}->getSortButton()->signal_connect('clicked', \&onSortClick, $self);
        if ($self->{movies})
        {
            $self->{movies}->reloadList;
            Glib::Timeout->add(100 ,\&showCurrent, $self);
            #$self->{moviesList}->showCurrent;
        }
    }
    
    sub showCurrent
    {
        my $self = shift;
        $self->{moviesList}->showCurrent;
    }
    
    sub allMovies
    {
        shift;
        my $self = shift;

        $self->{menubar}->selectAll;        
    }
    
    sub transformTitle
    {
        my ($self, $title) = @_;

        return $title if ! $self->{options}->transform;
        
        my $tmpTitle = $title;
        foreach (@{$self->{articles}})
        {
            return $tmpTitle if ($tmpTitle =~ s/^($_)(\s+|('))(.)(.*)/\U$4\E$5 ($1$3)/i); #'
        }
        return $tmpTitle;
    }
    
    sub new
    {
        my ($proto, $options, $searchJob) = @_;
        my $class = ref($proto) || $proto;
        my $self  = $class->SUPER::new('toplevel');
        bless ($self, $class);

        $options->setParent($self);
        $self->{options} = $options;

        $self->{searchJob} = $searchJob;

        $self->{options}->lang('EN') if ! $self->{options}->exists('lang');
        $self->{lang} = $GCLang::langs{$self->{options}->lang};
        
        if (($self->{options}->splash) || (! $self->{options}->exists('splash')))
        {
            my $splash = new GCSplashWindow($self);
        }
        else
        {
            $self->init;
            $self->loadPrevious;
        }

        return $self;
    }

    sub init
    {
        my $self = shift;

        $self->{options}->save if $self->checkImagesDirectory(1);
        
        $self->{agent} = 'Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.7.5) Gecko/20041111 Firefox/1.0';
        $self->{ignoreString} = 'gcf_000_ignore';
        $self->{imagePrefix} = 'gcfilms_';
        $self->{defaultImage} = $ENV{GCF_SHARE_DIR}.'/no.jpg';
        
        $self->{tooltips} = Gtk2::Tooltips->new();
        
        $self->{searchJob}->{command}->autoflush(1);
        $self->{searchJob}->{data}->autoflush(1);

        my $tmpTitle = $self->{options}->file ? basename($self->{options}->file) : '';
        $self->set_title('GCfilms - ['.$tmpTitle.']');
        $self->set_icon_from_file($ENV{GCF_SHARE_DIR}.'/icons/gcfilms_logo32.png');

        $self->signal_connect(delete_event => \&beforeDestroy, $self);
        $self->signal_connect(destroy => sub { Gtk2->main_quit; });
        
        $self->setMoviesList($self->{options}->view, $self->{options}->columns);

        $self->{panel} = new GCMoviePanel($self, $self->{options}, 0);
        $self->{scrollPanelMovie} = new Gtk2::ScrolledWindow;
        $self->{scrollPanelMovie}->set_policy ('automatic', 'automatic');
        $self->{scrollPanelMovie}->set_shadow_type('etched-in');
        $self->{scrollPanelMovie}->add_with_viewport($self->{panel});

        $self->{movies} = new GCMovies($self);
        $self->{movies}->setPanel($self->{panel});

        $self->{menubar} = new GCMenuBar($self);

        $self->{toolbar} = Gtk2::Toolbar->new();

        $self->{toolbar}->insert_stock("gtk-new" ,$self->{lang}->{NewMovieTooltip}, undef, \&newMovie, $self, -1);
        $self->{toolbar}->insert_stock("gtk-save" ,$self->{lang}->{SaveListTooltip}, undef, \&saveList, $self, -1);
        $self->{toolbar}->insert_stock("gtk-preferences" ,$self->{lang}->{PreferencesTooltip}, undef, \&options, $self, -1);

        $self->{toolbar}->append_space;

        $self->{toolbar}->insert_stock("gtk-find" ,$self->{lang}->{SearchTooltip}, undef, \&search, $self, -1);

        $self->{toolbar}->insert_stock("gtk-refresh" ,$self->{lang}->{ToolbarAllTooltip}, undef, \&allMovies, $self, -1);


        $self->{toolbar}->append_space;
                                                        
        # Movies list view selection /  Selection de la liste des films
		my %views = %{$self->{lang}->{OptionsViews}};
		if(scalar (keys %views) == 2)
		{
			# On cree le bouton toggle
			$self->{view} = new  Gtk2::ToggleButton->new( $views{$self->{options}->view} ) ;
			$self->{view}->show() ;
			$self->{toolbar}->append_widget($self->{view},
												$self->{lang}->{ViewTooltip},
                                                $self->{lang}->{ViewTooltip});   
			$self->{view}->set_active (!$self->{options}->view);
			$self->{view}->signal_connect( "toggled"	=> sub {
				my $togglebutton = $_[0];  
				my %views = %{$self->{lang}->{OptionsViews}};
				if ( $togglebutton->get_active )  
				{  
		    	  	# Si le controle arrive la, le bouton est baisse  
					$self->{options}->view(0);
					$togglebutton->set_label($views{0}); 
      			}  
	      		else  
	      		{  
					# Si le controle arrive la, le bouton est leve  
	      			$self->{options}->view(1);
					$togglebutton->set_label($views{1}); 
	     		}
				$self->setMoviesList($self->{options}->view, $self->{options}->columns);
 
	        });
		}
		else
		{
			$self->{viewOption} = new Gtk2::OptionMenu();
			$self->{view} = new Gtk2::Menu();
			my %views = %{$self->{lang}->{OptionsViews}};
			foreach (0..(scalar(keys %views) - 1))
			{
			    my $item = Gtk2::MenuItem->new_with_label($views{$_});
			    $self->{view}->append($item);
			}
	        $self->{viewOption}->set_menu($self->{view}) ;
	        $self->{viewOption}->signal_connect('changed' => sub {
	                return if $self->{viewOption}->child->get_label eq $views{$self->{options}->view};
		            if ($views{1} eq $self->{viewOption}->child->get_label)
	                {
	                    $self->{options}->view(1);
	                }
	                else
	                {
	                    $self->{options}->view(0);
	                }
	                $self->setMoviesList($self->{options}->view,
                                                             $self->{options}->columns);
		        });
	        
	        $self->{toolbar}->append_widget($self->{viewOption}, 
	                                                         $self->{lang}->{ViewTooltip},
	                                                         $self->{lang}->{ViewTooltip});                                                        
		 }
														  
        my $vbox = new Gtk2::VBox(0, 0);
        $self->{pane} = new Gtk2::HPaned;

        $self->{pane}->pack1($self->{moviesList},1,0);
        $self->{pane}->pack2($self->{scrollPanelMovie},1,1);
        $self->{pane}->set_position($self->{options}->split);

        $vbox->pack_start($self->{menubar}, 0, 0, 1);
        $vbox->pack_start($self->{toolbar},0,0,0 ) ;
        $vbox->pack_start($self->{pane}, 1, 1, 0);

        $self->{status} = Gtk2::Statusbar->new;
        $vbox->pack_start($self->{status},0,0,0 ) ;

        $self->add($vbox);

        $self->set_default_size($self->{options}->width,$self->{options}->height);

        $self->{resultsDialog} = new GCResultsDialog($self);
        $self->{optionsDialog} = new GCOptionsDialog($self, $self->{options});
        $self->{displayOptionsDialog} = new GCDisplayOptionsDialog($self);
        $self->{borrowersDialog} = new GCBorrowersDialog($self);
        $self->{borrowedDialog} = new GCBorrowedDialog($self);
        
        $self->{exportDialog} = new GCExportDialog($self);
        $self->{importDialog} = new GCImportDialog($self);
        
        $self->{searchDialog} = new GCSearchDialog($self);
        $self->{renameDialog} = new GCRenameDialog($self);
        $self->{importFieldsDialog} = new GCImportFieldsDialog($self);
        $self->{pluginsDialog} = new GCPluginsDialog($self);
        $self->{pluginsAskDialog} = new GCPluginsDialog($self);
        $self->{multiSiteDialog} = new GCMultiSiteDialog($self);

        $self->drag_dest_set('all', ["copy","private","default","move","link","ask"]);

        $self->signal_connect(drag_data_received => \&drop_handler);

        my $target_list = Gtk2::TargetList->new();
        my $atom1 = Gtk2::Gdk::Atom->new("text/uri-list");
        my $atom2 = Gtk2::Gdk::Atom->new("text/plain");
        $target_list->add($atom1, 0, 0);
        $target_list->add($atom2, 0, 0);

        $self->drag_dest_set_target_list($target_list);

        sub drop_handler {
            my ($treeview, $context, $widget_x, $widget_y, $data, $info,$time) = @_;
            my $type = $data->type->name;
            if ($type eq 'text/uri-list')
            {
                $self->newList;
                my @files = split /\n/, $data->data;
                $self->importWithDetect($_) foreach @files;
                $self->{movies}->setStatus;
            }
            elsif ((my $url = $data->data) =~ m/^http:\/\//)
            {
                $self->loadUrl($url);
            }
        }

        
        $self->show_all;

        $self->checkDisplayed;
        $self->checkPlugin;
        $self->checkBorrowers;
        $self->checkView;
        $self->checkTransform;
        $self->checkPanel;
        
        $self->{options}->searchStop(0) if ($^O =~ /win32/i);
    }

    sub checkDisplayed
    {
        my $self = shift;
        
        if ($self->{options}->status)
        {
            $self->{status}->show;
        }
        else
        {
            $self->{status}->hide;
        }

        if ($self->{options}->toolbar)
        {
            $self->{toolbar}->show;
            $self->{toolbar}->set_icon_size('small-toolbar') if $self->{options}->toolbar == 1;
            $self->{toolbar}->set_icon_size('large-toolbar') if $self->{options}->toolbar == 2;
        }
        else
        {
            $self->{toolbar}->hide;
        }

    }

    sub checkProxy
    {
		my $self= shift;
		
		$self->{plugin}->setProxy($self->{options}->proxy) if ($self->{plugin});
    }
    
    sub checkPlugin
    {
        my $self = shift;
        $self->{plugin} = undef;
        $self->{plugin} = $GCPlugins::pluginsMap{$self->{options}->plugin};

        $self->checkProxy;
    }
    
    sub checkBorrowers
    {
        my $self = shift;
        
        my @borrowers = split m/\|/, $self->{options}->borrowers;
        $self->{panel}->setBorrowers(\@borrowers);
    }
    
    sub checkView
    {
        my $self = shift;
		my %views = %{$self->{lang}->{OptionsViews}};
		if(scalar (keys %views) != 2) # inactif quand le toggle btn est utilis�
		{
        	$self->{view}->set_active($self->{options}->view);
        	$self->{viewOption}->remove_menu;
        	$self->{viewOption}->set_menu($self->{view});
		}
		else
		{
    		$self->{view}->set_label($views{$self->{options}->view});
    		$self->{view}->set_active (!$self->{options}->view);
		}
    }
    
    sub checkTransform
    {
        my $self = shift;
        
        my @array = split m/,/, $self->{options}->articles;
        
        foreach (@array)
        {
            s/^\s*//;
            s/\s*$//;
        }
        
        $self->{articles} = \@array;
        
        $self->{movies}->reloadList
    }
    
    sub checkFormats
    {
        my $self = shift;
        
        my @formats = split m/,/, $self->{options}->formats;
        $self->{panel}->setFormats(\@formats);
    }
    
    sub checkPanel
    {
        my $self = shift;

        $self->{panel}->setShowOption($self->{displayOptionsDialog}->{show});
    }
    
    sub checkUrlPicture
    {
        my $self = shift;
        
        $self->{panel}->setUrlType($self->{options}->urlPicture ? 'image' : 'text');
    }
    
    sub loadPrevious
    {
        my $self = shift;

        $self->{movies}->setOptions($self->{options});
        $self->{panel}->setTypes($self->{movies}->getTypes);
        $self->{panel}->setPlaces($self->{movies}->getPlaces);
        $self->{panel}->setAudios($self->{movies}->getAudios);
        $self->{panel}->setSubs($self->{movies}->getSubs);
        $self->{menubar}->setTypes($self->{movies}->getTypes);

        
        $self->{movies}->display(0);
        $self->{movies}->select(0);
    }

}

1;
