package GCMoviesLists;

###################################################
#
#  Copyright 2005 Tian
#
#  This file is part of GCfilms.
#
#  GCfilms is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  GCfilms is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with GCfilms; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###################################################

use strict;

{
    package GCTextList;
    
    use Gtk2::SimpleList;
    use base "Gtk2::ScrolledWindow";

    sub new
    {
        my ($proto, $parent, $title) = @_;
        my $class = ref($proto) || $proto;
        my $self  = $class->SUPER::new;
        bless ($self, $class);

        $self->{parent} = $parent;
        
        $self->set_policy ('automatic', 'automatic');
        $self->set_shadow_type('etched-in');
        
        $self->{list} = new Gtk2::SimpleList($title=>'text');
        $self->{list}->set_headers_clickable(1);
        $self->{list}->set_rules_hint(1);
        $self->{list}->get_column(0)->set_sort_indicator(1);
        
        $self->{list}->signal_connect (cursor_changed => sub {
            my ($sl, $path, $column) = @_;
            my @idx = $sl->get_selected_indices;
            $parent->display($idx[0]);
        });

        $self->{list}->signal_connect ('row-activated' => sub {
           $parent->displayInWindow;
        });

        $self->add($self->{list});
    
        return $self;
    }
    
    sub getSortButton
    {
        my $self = shift;
        $self->{list}->get_column(0);
    }
    
    sub setSortOrder
    {
        my ($self, $order) = @_;
        
        $self->{list}->get_column(0)->set_sort_order($order ? 'ascending' : 'descending');
        
        #Returns false because we don't need signal to be connected again.
        return 0;
    }

    sub reset
    {
        my $self = shift;
        
        @{$self->{list}->{data}} = ();
    }
    
    sub addMovie
    {
        my ($self, $info) = @_;
        
        push @{$self->{list}->{data}}, $self->{parent}->transformTitle($info->{title});
    }
    
    sub select
    {
        my ($self, $idx) = @_;
        
        $self->{list}->select($idx);
    }
    
    sub showCurrent
    {
        my $self = shift;
        
   		my $path = $self->{list}->get_selection->get_selected_rows;
		$self->{list}->scroll_to_cell($path) if $path;
    }
    
    sub changeCurrent
    {
        my $self = shift;
    }
    
}

{
    package GCImageList;
    
    use File::Basename;
    use base "Gtk2::VBox";

    sub new
    {
        my ($proto, $parent, $title, $columns) = @_;
        my $class = ref($proto) || $proto;
        my $self  = $class->SUPER::new;
        bless ($self, $class);
        
        $self->{scroll} = new Gtk2::ScrolledWindow;
        $self->{scroll}->set_policy ('automatic', 'automatic');
        $self->{scroll}->set_shadow_type('etched-in');
        
        $self->{parent} = $parent;
        
        $self->{sortButton} = Gtk2::Button->new_from_stock('gtk-sort-ascending');
        
        $self->{list} = new Gtk2::Table(0,$columns,1);
        $self->{tooltips} = Gtk2::Tooltips->new();
        
        $self->{columns} = $columns;
        $self->reset;

        my @colors = split m/,/, $parent->{options}->listBgColor;
        ($colors[0], $colors[1], $colors[2]) = (65535, 65535, 65535) if !@colors;
        $self->{inactiveBg} = new Gtk2::Gdk::Color($colors[0], $colors[1], $colors[2]);
        @colors = split m/,/, $parent->{options}->listFgColor;
        ($colors[0], $colors[1], $colors[2]) = (0, 0, 0) if !@colors;
        $self->{activeBg} = new Gtk2::Gdk::Color($colors[0], $colors[1], $colors[2]);

        $self->{defaultImage} = $parent->{defaultImage};
        
        $self->{list}->set_row_spacings(5);
        $self->{list}->set_col_spacings(5);
        $self->{list}->set_border_width(5);
        
        $self->{scroll}->add_with_viewport($self->{list});

        $self->{list}->parent->modify_bg('normal', $self->{inactiveBg});
        $self->{list}->parent->modify_bg('active', $self->{inactiveBg});
        $self->{list}->parent->modify_bg('prelight', $self->{inactiveBg});
        $self->{list}->parent->modify_bg('selected', $self->{inactiveBg});
        $self->{list}->parent->modify_bg('insensitive', $self->{inactiveBg});

        $self->pack_start($self->{sortButton},0,0,0);
        $self->pack_start($self->{scroll},1,1,0);
        
        $self->{list}->can_focus(1) if ($^O !~ /win32/i);
        
        return $self;
    }
    
    sub getSortButton
    {
        my $self = shift;

        return $self->{sortButton};
    }

    sub setSortOrder
    {
        my ($self, $order) = @_;
        
        $self->remove($self->{sortButton});
        $self->{sortButton}->destroy;
        $self->{sortButton} = Gtk2::Button->new_from_stock($order ? 'gtk-sort-ascending' : 'gtk-sort-descending');
        $self->pack_start($self->{sortButton},0,0,0);
        $self->reorder_child($self->{sortButton},0);
        $self->{sortButton}->show_all;
        
        #Returns true because we need signal to be connected again.
        return 1;
    }

    sub reset
    {
        my $self = shift;
        
        my @array = ();
        $self->{boxes} = \@array;
        $self->{number} = 0;
        $self->{current} = 0;
        
        my @children = $self->{list}->get_children;
        foreach (@children)
        {
            $self->{list}->remove($_);
            $_->destroy;
        }
    }
    
    sub addMovie
    {
        my ($self, $info) = @_;
        
        my $row = int $self->{number} / $self->{columns};
        my $col = $self->{number} % $self->{columns};
        
        $self->{list}->resize($row + 1, $self->{columns});

        my $eventBox = new Gtk2::EventBox;
        $eventBox->can_focus(1) if ($^O !~ /win32/i);
                
        my $image = new Gtk2::Image;
        my $displayedImage = $info->{image};
        if (File::Spec->file_name_is_absolute($displayedImage))
        {
            $displayedImage = $self->{defaultImage} if (! -e $displayedImage);
        }
        else
        {            
            my $dir;
            $dir = '.';
            $dir = dirname($self->{parent}->{options}->file)
                if $self->{parent}->{options}->file;
            if (-f $dir.'/'.$displayedImage)
            {
                $displayedImage = $dir.'/'.$displayedImage;
            }
            else
            {
                $displayedImage = $self->{defaultImage} unless (-e $displayedImage);
            }
        }
        $displayedImage = $self->{defaultImage} if ! -f $displayedImage;
        my $pixbuf = Gtk2::Gdk::Pixbuf->new_from_file($displayedImage);
        $pixbuf = $pixbuf->scale_simple(120,160,'bilinear') if ($pixbuf->get_width > 120) || ($pixbuf->get_height > 160);
        $image->set_from_pixbuf($pixbuf);
        $eventBox->add($image);
        $eventBox->set_size_request(130,170);
        
        my $i = $self->{number};
        $self->{tooltips}->set_tip($eventBox, $info->{title}, '');
        $eventBox->signal_connect('button_press_event' => sub {
                my ($widget, $event) = @_;
                $self->select($i);
                $self->{parent}->display($i) unless $event->type eq '2button-press';
                $self->{parent}->displayInWindow if $event->type eq '2button-press';
                $widget->grab_focus;
            });
        $eventBox->signal_connect('key-press-event' => sub {
                my ($widget, $event) = @_;
                my $idx = $i;
                my $key = Gtk2::Gdk->keyval_name($event->keyval);
                if (($key eq 'Return') || ($key eq 'space'))
                {
                    $self->{parent}->displayInWindow;
                    return 1;
                }
                $idx++ if $key eq 'Right';
                $idx-- if $key eq 'Left';
                $idx += $self->{columns} if $key eq 'Down';
                $idx -= $self->{columns} if $key eq 'Up';
                return 1 if ($idx < 0) || ($idx >= scalar @{$self->{boxes}});
                $self->select($idx);
                $self->{parent}->display($idx);
                $self->{boxes}->[$idx]->grab_focus;
                $self->showCurrent;
                return 1;
                
            });
        $self->{list}->attach($eventBox, $col, $col+1, $row, $row+1, 'fill', 'fill', 0, 0);
        $eventBox->modify_bg('normal', $self->{inactiveBg});
        $eventBox->show_all;
        
        push @{$self->{boxes}}, $eventBox;
        
        $self->{number}++;
    }
    
    sub select
    {
        my ($self, $idx) = @_;
        
        my @boxes = @{$self->{boxes}};
        
        return if ! scalar(@boxes);
        
        $boxes[$self->{current}]->modify_bg('normal', $self->{inactiveBg});
        $self->{current} = $idx;
        $boxes[$self->{current}]->modify_bg('normal', $self->{activeBg});
    }
    
    sub showCurrent
    {
        my $self = shift;
 
        my $adj = $self->{scroll}->get_vadjustment;
        my $totalRows = int $self->{number} / $self->{columns};
        my $row = (int $self->{current} / $self->{columns}) - 1;

        $adj->set_value((($adj->upper - $adj->lower) / $totalRows) * $row) if $totalRows > 0;
    }
    
    sub changeCurrent
    {
        my ($self, $image) = @_;
        
        my @boxes = @{$self->{boxes}};
        
        my $pixbuf = Gtk2::Gdk::Pixbuf->new_from_file($image);
        $pixbuf = $pixbuf->scale_simple(120,160,'bilinear') if ($pixbuf->get_width > 120) || ($pixbuf->get_height > 160);
        $boxes[$self->{current}]->child->set_from_pixbuf($pixbuf);
    }

    
}

1;