package GCPlugins::GCDVDfr;

###################################################
#
#  Copyright 2005 Tian
#
#  This file is part of GCfilms.
#
#  GCfilms is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  GCfilms is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with GCfilms; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###################################################

###################################
#														   #
#			Plugin soumis par MeV			   #
#														   #
###################################

use strict;
use utf8;

use GCPlugins::GCPluginsBase;

{
    package GCPlugins::GCPluginDVDfr;

    use base qw(GCPluginParser);

    sub start
    {
        my ($self, $tagname, $attr, $attrseq, $origtext) = @_;
	
        $self->{inside}->{$tagname}++;
				
        if ($self->{parsingList})
        {
            if ($tagname eq "a")
            {
                if (($attr->{href} =~ /^\.\.\/dvd\/dvd\.php\?id=/))
                {
                    my $url = $attr->{href};
                    $self->{isMovie} = 1;
                    $self->{isInfo} = 1;
                    $self->{movieIdx}++;
                    $self->{movieList}[$self->{movieIdx}]->{"url"} = $url;
                }
            }
			elsif ($tagname eq "td")
			{
					if (($attr->{class} eq "searchText") && ($attr->{align} eq "center"))
					{
							$self->{isInfo}=1;
					}
			}
        }
        else
        {
            if ($tagname eq "img")
            {
                if ($attr->{src} =~ /^\.\.\/images\/dvd\/cover_200x280\//)
                {
                    $self->{curInfo}->{image} = "http://dvdfr.com/dvd/" . $attr->{src};
                }
            }
            elsif ($tagname eq "div")
            {
                $self->{insideName} = 1 if $attr->{class} eq "dvd_title";
                $self->{insideOrig} = 1 if $attr->{class} eq "dvd_titlevo";
                $self->{insideNat} = 1 if $attr->{class} eq "dvd_titleinfo";
                $self->{insideTime} = 1 if $attr->{class} eq "duree";
            }
        }
    }

    sub end
    {
		my ($self, $tagname) = @_;
		
        $self->{inside}->{$tagname}--;
    }

    sub text
    {
        my ($self, $origtext) = @_;

        return if length($origtext) < 2;
           
        if ($self->{parsingList})
        {
            if ($self->{isMovie})
            {
                $self->{movieList}[$self->{movieIdx}]->{"title"} = $origtext;
                $self->{isMovie} = 0;
                $self->{isInfo} = 1;
                return;
            }
            elsif ($self->{isInfo})
            {
                if (($origtext =~ /[0-9]{4}/) || ($origtext eq " "))
                {
                    $self->{movieList}[$self->{movieIdx}]->{"year"} = $origtext;
                }
                elsif ($origtext =~ /\w+\s+\w+/)
                {
                    $self->{movieList}[$self->{movieIdx}]->{"director"} = $origtext;
                }
                $self->{isInfo} = 0;
            }
        }
        else
        {
            $origtext =~ s/\s{2,}//g;
            
            if ($self->{insideName})
            {
                $self->{curInfo}->{title} = $self->capWord($origtext);
                $self->{insideName} = 0;
            }
            elsif ($self->{insideDirector})
            {
                $self->{curInfo}->{director} = $origtext if !$self->{curInfo}->{director};
                $self->{insideDirector} = 0;
            }
            elsif ($self->{insideActors})
            {
                $self->{curInfo}->{actors} = $origtext if !$self->{curInfo}->{actors};
                $self->{insideActors} = 0;
            }
            elsif ($self->{insideOrig})
            {
                $self->{curInfo}->{orig} = $origtext;
                $self->{insideOrig} = 0;
            }
            if ($self->{insideNat})
            {
                $origtext =~ /(.*?) , ([0-9]{4})/;
				$self->{curInfo}->{nat} = $1;
				$self->{curInfo}->{date} = $2;
				$self->{insideNat} = 0;
            }
            elsif ($self->{insideSynopsis})
            {
                $self->{curInfo}->{synopsis} = $origtext;
                $self->{insideSynopsis} = 0;
            }
            elsif ($self->{insideTime})
            {
                $self->{curInfo}->{time} = $origtext;
                $self->{insideTime} = 0;
            }
            elsif ($self->{inside}->{div})
            {
                $self->{insideSynopsis} = 1 if $origtext =~ m/Synopsis/;
                $self->{insideDirector} = 1 if $origtext =~ m/R.alisation/;
                $self->{insideActors} = 1 if $origtext =~ m/Avec\.\.\./;
           }
        }
    } 

    sub new
    {
        my $proto = shift;
        my $class = ref($proto) || $proto;
        my $self  = $class->SUPER::new();
        bless ($self, $class);

        $self->{isInfo} = 0;
        $self->{isMovie} = 0;
        $self->{curName} = undef;
        $self->{curUrl} = undef;

        return $self;
    }

    sub preProcess
    {
        my ($self, $html) = @_;

        $html =~ s/&nbsp;/ /g;
        $html =~ s/<br> /, /g;
        $html =~ s/<a class="dvd_text" href="\.\.\/search\/search\.php\?star=[0-9]*&startype=[0-9]">([^<]*)<\/a>/$1/gi;
        $html =~ s/<div class="dvd_text">([^<]*)<\/div>/$1/gi;
        $html =~ s/<td><img alt="Dur.e" title="Dur.e" src="\.\.\/images\/duree\.gif" width="32" height="14" align="absmiddle"><\/td>[^<]*<td>([0-9]* min)<\/td>/<td><div class="duree">$1<\/div><\/td>/;
        
        return $html;
    }
    
    sub getSearchUrl
    {
		my ($self, $word) = @_;
	
        return "http://dvdfr.com/search/search.php?multiname=$word";
    }
    
    sub getMovieUrl
    {
		my ($self, $url) = @_;
		
        return "http://dvdfr.com/dvd/" . $url;
    }

    sub getName
    {
        return "Dvdfr.com";
    }
    
    sub getImageFile
    {
        return "dvdfr.png";
    }

    sub getAuthor
    {
        return 'MeV';
    }
    
    sub getLang
    {
        return 'FR';
    }

}

1;
