package GCPlugins::GCImdb;

###################################################
#
#  Copyright 2005 Tian
#
#  This file is part of GCfilms.
#
#  GCfilms is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  GCfilms is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with GCfilms; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###################################################

use strict;

use GCPlugins::GCPluginsBase;

{
    package GCPlugins::GCPluginImdb;

    use base qw(GCPluginParser);
 
    sub start
    {
        my ($self, $tagname, $attr, $attrseq, $origtext) = @_;
	
        $self->{inside}->{$tagname}++;

        if ($self->{parsingEnded})
        {
            if ($tagname eq "a")
            {
                if ($attr->{href} =~ m/mymovies\/list\?pending\&add=([0-9]*)/)
                {
                    $self->{movieIdx} = 0;
                    $self->{movieList}[0]->{"url"} = '/title/tt'.$1.'/';
                }
            }
            return;
        }
        
        if ($self->{parsingList})
        {
            if ($tagname eq "a")
            {

                if ($attr->{href} =~ /^\/title\//)
                {
                    my $url = $attr->{href};
                    $self->{isMovie} = 1;
                    $self->{isInfo} = 1;
                    $self->{movieIdx}++;
                    $self->{movieList}[$self->{movieIdx}]->{"url"} = $url;
                }
            }
        }
        else
        {
            if ($tagname eq "img")
            {
                $self->{curInfo}->{image} = $attr->{src} if $attr->{alt} eq 'cover';
            }
            elsif ($tagname eq "a")
            {
                if ($attr->{href} =~ m/fullcredits/)
                {
                    $self->{insideActors} = 0;
                    $self->{curInfo}->{actors} =~ s/, $//;
                }
                $self->{insideSynopsis} = 0 if ($attr->{href} =~ m/plotsummary/);
                $self->{insideType} = 0 if ($attr->{href} =~ m/keywords/);
            }
        }
    }

    sub end
    {
		my ($self, $tagname) = @_;
		
		$self->{inside}->{$tagname}--;

    }

    sub text
    {
        my ($self, $origtext) = @_;

        return if length($origtext) < 2;
        
        $origtext =~ s/&#34;/"/g;
        $origtext =~ s/&#179;/3/g;
        $origtext =~ s/&#[0-9]*;//g;
        $origtext =~ s/\n//g;

        return if ($self->{parsingEnded});
        
        if ($self->{parsingList})
        {
            if (($self->{inside}->{h1}) && ($origtext !~ m/IMDb title search/))
            {
                $self->{parsingEnded} = 1;
            }
            if ($self->{isMovie})
            {
                $self->{movieList}[$self->{movieIdx}]->{"title"} = $origtext;
                $self->{isMovie} = 0;
                $self->{isInfo} = 1;
                return;
            }
            if ($self->{isInfo})
            {
                $self->{movieList}[$self->{movieIdx}]->{"year"} = $1 if $origtext =~ m/\(([0-9]*)\)/;
                $self->{isInfo} = 0;
            }
        }
        else
        {
            $self->{insideNat} = 0 if $origtext =~ m/Langue:/;
            $self->{insideOtherTitles} = 0 if $origtext =~ m/Dure:/;
            $self->{insideSynopsis} = 0 if $origtext =~ m/(Avis des utilisateurs)|(Note des utilisateurs)/;
            $self->{insideType} = 0 if $origtext =~ m/(Note des utilisateurs:)|(Synopsis:)/;

            if ($self->{insideType})
            {
                $self->{curInfo}->{type} .= lc($origtext).',' if $origtext !~ m/\//;
            }
            elsif ($self->{insideDirector})
            {
                $self->{curInfo}->{director} = $origtext;
                $self->{insideDirector} = 0;
            }
            elsif ($self->{insideSynopsis})
            {
                $self->{curInfo}->{synopsis} .= $origtext;
            }
            elsif ($self->{insideNat})
            {
                $self->{curInfo}->{nat} .= $origtext;
            }
            elsif ($self->{insideTime})
            {
                $self->{curInfo}->{time} = $origtext;
                $self->{curInfo}->{time} =~ s/.*?://;
                $self->{insideTime} = 0;
            }
            elsif (($self->{insideActors}) && ($self->{inside}->{a}))
            {
                $self->{curInfo}->{actors} .= $origtext.', ' if ($self->{actorsCounter} < $GCPluginParser::MAX_ACTORS);
                $self->{actorsCounter}++;
            }
            elsif ($self->{insideOtherTitles})
            {
                if ($origtext =~ m/(.*?) \(International.*/)
                {
                    $self->{curInfo}->{title} = $1;
                    $self->{insideOtherTitles} = 0;
                }
            }
            
            if ($self->{inside}->{h1})
            {
                $self->{curInfo}->{date} = $origtext if $origtext =~ m/[0-9]+/;
                $self->{curInfo}->{orig} = $origtext if !$self->{curInfo}->{orig};
            }
            elsif ($self->{inside}->{b})
            {
                $self->{insideDirector} = 1 if $origtext =~ m/Ralis par/;
                $self->{insideType} = 1 if $origtext =~ m/Genre:/;
                $self->{insideSynopsis} = 1 if ($origtext =~ m/Synopsis:|Plot Summary:/);
                $self->{insideOtherTitles} = 1 if ($origtext =~ m/Autre\(s\) titre\(s\) :/);
                $self->{insideTime} = 1 if $origtext =~ m/Dure:/;
                $self->{insideNat} = 1 if $origtext =~ m/Pays:/;
                $self->{insideActors} = 1 if $origtext =~ m/Cast overview|credited cast:/i;
            }
            
        }
    }

    sub new
    {
        my $proto = shift;
        my $class = ref($proto) || $proto;
        my $self  = $class->SUPER::new();
        bless ($self, $class);

        $self->{isInfo} = 0;
        $self->{isMovie} = 0;
        $self->{curName} = undef;
        $self->{curUrl} = undef;

        return $self;
    }

    sub preProcess
    {
        my ($self, $html) = @_;
        
        $self->{parsingEnded} = 0;
        
        return $html;
    }

    sub getSearchUrl
    {
		my ($self, $word) = @_;
	
		return "http://french.imdb.com/find?q=$word;tt=on;mx=20";
    }
    
    sub getMovieUrl
    {
		my ($self, $url) = @_;
		
		return "http://french.imdb.com".$url;
    }

    sub getName
    {
        return "IMDb";
    }
    
    sub getImageFile
    {
        return "imdb.png";
    }

    sub getAuthor
    {
        return 'Tian';
    }
    
    sub getLang
    {
        return 'EN';
    }
}

1;
